@@ -3,7 +3,7 @@ use crate::format::{Error, Format as _, RegionOptions, Result};
 use crate::items::macros::Macro;
 use crate::items::tokens::{CommentKind, CommentToken};
 use crate::span::{Position, Span};
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashSet};
 
 #[derive(Debug)]
 pub struct Formatter {
@@ -11,7 +11,7 @@ pub struct Formatter {
     text: String,
     macros: BTreeMap<Position, Macro>,
     comments: BTreeMap<Position, CommentToken>,
-    macro_state: MacroState,
+    formatting_macros: HashSet<Position>,
 }
 
 impl Formatter {
@@ -26,18 +26,25 @@ impl Formatter {
             text,
             macros,
             comments,
-            macro_state: MacroState::None,
+            formatting_macros: HashSet::new(),
         }
     }
 
     pub fn finish(mut self) -> Result<String> {
-        let eof = crate::items::module::Eof {
-            position: Position::new(usize::MAX, usize::MAX, usize::MAX),
-        };
+        let eof = Position::new(usize::MAX, usize::MAX, usize::MAX);
         self.write_comments_and_macros(&eof, None)?;
         Ok(self.writer.formatted_text().to_owned())
     }
 
+    pub fn check_trailing_columns(&self) -> Result<()> {
+        let too_long = self.current_column() + self.region_config().trailing_columns
+            > self.region_config().max_columns;
+        if too_long && !self.region_config().allow_too_long_line {
+            return Err(Error::LineTooLong);
+        }
+        Ok(())
+    }
+
     pub fn current_column(&self) -> usize {
         if self.writer.last_char() == '\n' {
             self.writer.config().indent
@@ -50,10 +57,18 @@ impl Formatter {
         self.current_column() - self.writer.config().indent
     }
 
+    pub fn parent_indent(&self) -> usize {
+        self.writer.parent_indent()
+    }
+
     pub fn region_config(&self) -> &RegionConfig {
         self.writer.config()
     }
 
+    pub fn last_char(&self) -> char {
+        self.writer.last_char()
+    }
+
     pub fn is_multi_line_allowed(&self) -> bool {
         self.writer.config().allow_multi_line
     }
@@ -63,16 +78,12 @@ impl Formatter {
     }
 
     pub fn write_newline(&mut self) -> Result<()> {
-        if !matches!(self.macro_state, MacroState::Written(_)) {
-            self.writer.write_newline()?;
-        }
+        self.writer.write_newline()?;
         Ok(())
     }
 
     pub fn write_space(&mut self) -> Result<()> {
-        if !matches!(self.macro_state, MacroState::Written(_)) {
-            self.writer.write_space()?;
-        }
+        self.writer.write_space()?;
         Ok(())
     }
 
@@ -80,17 +91,10 @@ impl Formatter {
         self.write_comments_and_macros(item, Some(CommentKind::Trailing))?;
         self.write_comments_and_macros(item, Some(CommentKind::Post))?;
         self.writer.write_item(&self.text, item)?;
-        match self.macro_state {
-            MacroState::Written(end) if end < item.end_position() => {
-                self.macro_state = MacroState::None;
-            }
-            _ => {}
-        }
         Ok(())
     }
 
     fn write_comment(&mut self, item: &CommentToken) -> Result<()> {
-        assert_eq!(self.macro_state, MacroState::None);
         self.writer.write_comment(&self.text, item)?;
         Ok(())
     }
@@ -122,7 +126,9 @@ impl Formatter {
     }
 
     fn write_macro(&mut self, item: &Macro) -> Result<()> {
-        self.macro_state = MacroState::Writing(item.start_position());
+        assert!(!self.formatting_macros.contains(&item.start_position()));
+
+        self.formatting_macros.insert(item.start_position());
         let result = (|| {
             let start_offset = item.start_position().offset();
             if matches!(
@@ -138,13 +144,10 @@ impl Formatter {
             if matches!(self.text.as_bytes().get(end_offset), Some(b' ' | b'\n')) {
                 self.write_space()?;
             }
+            self.writer.skip_whitespace_until(item.end_position());
             Ok(())
         })();
-        if result.is_err() {
-            self.macro_state = MacroState::None;
-        } else {
-            self.macro_state = MacroState::Written(item.end_position());
-        }
+        self.formatting_macros.remove(&item.start_position());
         result
     }
 
@@ -171,9 +174,6 @@ impl Formatter {
                     break;
                 }
             } else {
-                if self.macro_state == MacroState::Writing(macro_start.expect("unreachable")) {
-                    break;
-                }
                 let macro_call = self.macros[&macro_start.unwrap()].clone();
                 self.write_macro(&macro_call)?;
             }
@@ -193,6 +193,7 @@ impl Formatter {
         self.macros
             .range(self.next_position()..)
             .map(|x| x.0)
+            .filter(|x| !self.formatting_macros.contains(x))
             .copied()
             .next()
     }
@@ -201,10 +202,3 @@ impl Formatter {
         self.writer.next_position()
     }
 }
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-enum MacroState {
-    None,
-    Writing(Position),
-    Written(Position),
-}
