@@ -6,7 +6,7 @@ use crate::items::generics::{Clauses, Items, Maybe, Parenthesized};
 use crate::items::keywords::{EndKeyword, FunKeyword};
 use crate::items::styles::{RightSpace, Space};
 use crate::items::symbols::{ColonSymbol, RightArrowSymbol, SlashSymbol};
-use crate::items::tokens::{AtomToken, IntegerToken, VariableToken};
+use crate::items::tokens::VariableToken;
 use crate::parse::Parse;
 use crate::span::Span;
 
@@ -15,7 +15,6 @@ pub enum FunctionExpr {
     Defined(Box<DefinedFunctionExpr>),
     Anonymous(Box<AnonymousFunctionExpr>),
     Named(Box<NamedFunctionExpr>),
-    NameAndArity(Box<NameAndArity<AtomToken, IntegerToken>>), // For attributes such as `-export`
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
