@@ -9,18 +9,15 @@ pub struct StringExpr(MaybeRepeat<StringToken>);
 
 impl Format for StringExpr {
     fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        fmt.with_subregion(
-            format::RegionOptions::new().indent(format::IndentMode::CurrentColumn),
-            |fmt| {
-                for (i, item) in self.0.get().iter().enumerate() {
-                    fmt.format_item(item)?;
-                    if i + 1 < self.0.get().len() {
-                        fmt.needs_newline();
-                    }
+        fmt.subregion().current_column_as_indent().enter(|fmt| {
+            for (i, item) in self.0.get().iter().enumerate() {
+                item.format(fmt)?;
+                if i + 1 < self.0.get().len() {
+                    fmt.write_newline()?;
                 }
-                Ok(())
-            },
-        )
+            }
+            Ok(())
+        })
     }
 }
 
@@ -28,15 +25,8 @@ impl Format for StringExpr {
 mod tests {
     use crate::items::expressions::Expr;
 
-    fn format(text: &str) -> String {
-        crate::FormatOptions::<crate::items::styles::Child<Expr>>::new()
-            .max_columns(20)
-            .format_text(text)
-            .expect("parse or format failed")
-    }
-
     #[test]
-    fn tuple_works() {
+    fn string_works() {
         let texts = [
             "\"foo\"",
             indoc::indoc! {r#"
@@ -45,10 +35,11 @@ mod tests {
                 "baz""#},
             indoc::indoc! {r#"
                 foo("bar"
-                    "baz", qux)"#},
+                    "baz",
+                    qux)"#},
         ];
         for text in texts {
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Expr);
         }
     }
 }
