@@ -1,34 +1,39 @@
 use crate::format::{self, Format, Formatter};
 use crate::items::expressions::Expr;
-use crate::items::generics::{Either, Items, Maybe, Parenthesized};
+use crate::items::generics::{Args2, Either, Maybe};
 use crate::items::symbols::{
     CloseParenSymbol, CommaSymbol, DotSymbol, OpenParenSymbol, QuestionSymbol,
 };
-use crate::items::tokens::{AtomToken, Token, VariableToken};
+use crate::items::tokens::{AtomToken, StringToken, Token, VariableToken};
 use crate::parse::{self, Parse, TokenStream};
 use crate::span::{Position, Span};
 use erl_tokenize::values::{Keyword, Symbol};
 use std::collections::HashMap;
 
+/// `?` `$NAME` (`(` (`$ARG` `,`?)* `)`)?
+///
+/// - $NAME: [AtomToken] | [VariableToken]
+/// - $ARG: [Token]+
 #[derive(Debug, Clone, Span, Format)]
 pub struct Macro {
     question: QuestionSymbol,
     name: MacroName,
-    args: Maybe<Parenthesized<Items<MacroArg>>>,
+    args: Maybe<MacroArgs>,
 }
 
 impl Macro {
-    pub fn parse(
+    // TODO: resume_parse
+    pub(crate) fn parse(
         ts: &mut TokenStream,
         question: QuestionSymbol,
         name: MacroName,
-        arity: Option<usize>,
+        has_args: bool,
     ) -> parse::Result<Self> {
-        if let Some(_arity) = arity {
+        if has_args {
             Ok(Self {
                 question,
                 name,
-                args: ts.parse()?,
+                args: Maybe::from_item(ts.parse()?),
             })
         } else {
             Ok(Self {
@@ -39,40 +44,51 @@ impl Macro {
         }
     }
 
-    pub fn expand(
-        &self,
-        variables: Option<Vec<VariableToken>>,
-        replacement: Vec<Token>,
-    ) -> Vec<Token> {
-        let args = if let (Some(vars), Some(vals)) =
-            (&variables, self.args.get().map(|x| x.get().get()))
+    pub fn arity(&self) -> Option<usize> {
+        self.args.get().map(|x| x.get().len())
+    }
+
+    pub fn expand(&self, variables: Option<Vec<String>>, replacement: Vec<Token>) -> Vec<Token> {
+        let args = if let (Some(vars), Some(vals)) = (&variables, self.args.get().map(|x| x.get()))
         {
             vars.iter()
-                .map(|x| x.value())
+                .map(|x| &x[..])
                 .zip(vals.iter())
                 .collect::<HashMap<_, _>>()
         } else {
             HashMap::new()
         };
 
+        let mut do_stringify = false;
         let mut tokens = Vec::new();
         for token in replacement {
             match token {
+                Token::Variable(x) if do_stringify => {
+                    let dummy =
+                        StringToken::new("EFMT_DUMMY", x.start_position(), x.end_position());
+                    tokens.push(dummy.into());
+                }
                 Token::Variable(x) if args.contains_key(x.value()) => {
                     tokens.extend(args[x.value()].tokens().iter().cloned());
                 }
+                Token::Symbol(x) if x.value() == Symbol::DoubleQuestion => {
+                    do_stringify = true;
+                    continue;
+                }
                 token => {
                     tokens.push(token);
                 }
             }
+            do_stringify = false;
         }
+        // TODO: delete(?)
         tokens.iter_mut().for_each(|token| token.set_span(self));
         tokens
     }
 }
 
 #[derive(Debug, Clone, Span, Parse, Format)]
-pub struct MacroName(Either<AtomToken, VariableToken>);
+pub(crate) struct MacroName(Either<AtomToken, VariableToken>);
 
 impl MacroName {
     pub fn value(&self) -> &str {
@@ -84,7 +100,7 @@ impl MacroName {
 }
 
 #[derive(Debug, Clone)]
-pub struct MacroReplacement {
+pub(crate) struct MacroReplacement {
     tokens: Vec<Token>,
     expr: Option<Expr>, // The expression representation of `tokens` (for formatting)
     start_position: Position,
@@ -112,18 +128,20 @@ impl Span for MacroReplacement {
 
 impl Parse for MacroReplacement {
     fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
-        let start_position = ts.current_whitespace_token()?.end_position();
-        let expr = ts
-            .peek::<(Expr, (CloseParenSymbol, DotSymbol))>()
-            .map(|(expr, _)| expr);
-        let mut tokens = Vec::new();
-        while ts.peek::<(CloseParenSymbol, DotSymbol)>().is_none() {
-            tokens.push(ts.parse()?);
-        }
-        Ok(Self {
-            tokens,
-            expr,
-            start_position,
+        ts.enter_macro_replacement(|ts| {
+            let start_position = ts.current_whitespace_token()?.end_position();
+            let expr = ts
+                .peek::<(Expr, (CloseParenSymbol, DotSymbol))>()
+                .map(|(expr, _)| expr);
+            let mut tokens = Vec::new();
+            while ts.peek::<(CloseParenSymbol, DotSymbol)>().is_none() {
+                tokens.push(ts.parse()?);
+            }
+            Ok(Self {
+                tokens,
+                expr,
+                start_position,
+            })
         })
     }
 }
@@ -139,7 +157,48 @@ impl Format for MacroReplacement {
 }
 
 #[derive(Debug, Clone)]
-pub struct MacroArg {
+struct Empty;
+
+impl Span for Empty {
+    fn start_position(&self) -> Position {
+        unreachable!()
+    }
+
+    fn end_position(&self) -> Position {
+        unreachable!()
+    }
+}
+
+impl Parse for Empty {
+    fn parse(ts: &mut parse::TokenStream) -> parse::Result<Self> {
+        let token = ts.parse()?;
+        Err(parse::Error::unexpected_token(ts, token))
+    }
+}
+
+impl Format for Empty {
+    fn format(&self, _: &mut format::Formatter) -> format::Result<()> {
+        unreachable!()
+    }
+}
+
+#[derive(Debug, Clone, Span, Parse, Format)]
+enum MacroArgs {
+    Empty(Args2<Empty>),
+    Args(Args2<MacroArg>),
+}
+
+impl MacroArgs {
+    fn get(&self) -> &[MacroArg] {
+        match self {
+            Self::Empty(_) => &[],
+            Self::Args(x) => x.get(),
+        }
+    }
+}
+
+#[derive(Debug, Clone)]
+struct MacroArg {
     tokens: Vec<Token>,
     expr: Option<Expr>, // The expression representation of `tokens` (for formatting)
 }
@@ -183,19 +242,9 @@ impl Parse for MacroArg {
         let mut level = Level::default();
         while tokens.is_empty()
             || !level.is_toplevel()
-            || ts
-                .peek::<Either<CommaSymbol, CloseParenSymbol>>()
-                .filter(|t| !ts.macros().contains_key(&t.start_position()))
-                .is_none()
+            || ts.peek::<Either<CommaSymbol, CloseParenSymbol>>().is_none()
         {
             let token: Token = ts.parse()?;
-
-            let is_macro_expanded = ts.macros().contains_key(&token.start_position());
-            if is_macro_expanded {
-                tokens.push(token);
-                continue;
-            }
-
             match &token {
                 Token::Symbol(x) => match x.value() {
                     Symbol::OpenParen => {
@@ -203,7 +252,7 @@ impl Parse for MacroArg {
                     }
                     Symbol::CloseParen => {
                         if level.paren == 0 {
-                            todo!()
+                            todo!("{:?}", ts.current_position());
                         }
                         level.paren -= 1;
                     }
@@ -212,7 +261,7 @@ impl Parse for MacroArg {
                     }
                     Symbol::CloseBrace => {
                         if level.brace == 0 {
-                            todo!();
+                            todo!("{:?}", ts.current_position());
                         }
                         level.brace -= 1;
                     }
@@ -221,7 +270,7 @@ impl Parse for MacroArg {
                     }
                     Symbol::CloseSquare => {
                         if level.square == 0 {
-                            todo!();
+                            todo!("{:?}", ts.current_position());
                         }
 
                         level.square -= 1;
@@ -231,14 +280,18 @@ impl Parse for MacroArg {
                     }
                     Symbol::DoubleRightAngle => {
                         if level.bits == 0 {
-                            todo!();
+                            todo!("{:?}", ts.current_position());
                         }
                         level.bits -= 1;
                     }
                     _ => {}
                 },
                 Token::Keyword(x) => match x.value() {
-                    Keyword::Begin | Keyword::Try | Keyword::Case | Keyword::If => {
+                    Keyword::Begin
+                    | Keyword::Try
+                    | Keyword::Case
+                    | Keyword::If
+                    | Keyword::Receive => {
                         level.block += 1;
                     }
                     Keyword::Fun => {
@@ -250,7 +303,7 @@ impl Parse for MacroArg {
                     }
                     Keyword::End => {
                         if level.block == 0 {
-                            todo!();
+                            todo!("{:?}", ts.current_position());
                         }
                         level.block -= 1;
                     }
@@ -295,13 +348,31 @@ mod tests {
                 ?FOO.
             "},
             indoc::indoc! {"
+            -define(FOO, bar).
+            baz() ->
+                ?FOO().
+            "},
+            indoc::indoc! {"
+            -define(FOO, bar).
+            -define(FOO(),
+                    bar bar).
+            baz() ->
+                ?FOO.
+            "},
+            indoc::indoc! {"
+            baz() ->
+                ?UNKNOWN.
+            "},
+            indoc::indoc! {"
             -define(INC, 1 +).
             inc(A) ->
                 ?INC A.
             "},
             indoc::indoc! {"
-            -define(FOO_OPEN, foo().
-            -define(FOO_CLOSE, )).
+            -define(FOO_OPEN,
+                    foo().
+            -define(FOO_CLOSE,
+                    )).
             foo(A) ->
                 ?FOO_OPEN A ?FOO_CLOSE.
             "},
@@ -320,17 +391,24 @@ mod tests {
     #[test]
     fn macro_with_args_works() {
         let texts = [
-            // TODO: Should have a newline after ',' if the line is too long.
             indoc::indoc! {"
-            -define(FOO(Bar), {Bar,
-                               Baz}).
+            %---10---|%---20---|
+            -define(FOO(), foo).
+            foo() ->
+                ?FOO().
+            "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(FOO(Bar),
+                    {Bar, Baz}).
             qux() ->
                 ?FOO(quux).
             "},
             indoc::indoc! {"
+            %---10---|%---20---|
             -define(FOO(Bar,
-                        Baz), {Bar,
-                               Baz}).
+                        Baz),
+                    {Bar, Baz}).
             qux() ->
                 ?FOO(begin
                          foo,
@@ -340,7 +418,7 @@ mod tests {
                      hello).
             "},
             indoc::indoc! {"
-            -define(FOO(Bar), Bar).
+            -define(FOO(A), A).
 
             qux() ->
                 [?FOO(begin
@@ -359,6 +437,20 @@ mod tests {
             main() ->
                 ?baz(?bar(?foo)).
             "},
+            indoc::indoc! {"
+            -define(Foo(A),
+                    ??A).
+            bar() ->
+                ?Foo(10).
+            "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(FOO(), foo).
+            -define(FOO,
+                    foo foo).
+            foo() ->
+                ?FOO() + 10.
+            "},
         ];
         for text in texts {
             crate::assert_format!(text, Module);
@@ -383,7 +475,8 @@ mod tests {
                 ?bar(?foo) [c].
             "},
             indoc::indoc! {"
-            -define(a, [1, 2, 3], [).
+            -define(a,
+                    [1, 2, 3], [).
             -define(b(A), A).
 
             main() ->
@@ -394,4 +487,77 @@ mod tests {
             crate::assert_format!(text, Module);
         }
     }
+
+    #[test]
+    fn macro_and_comment_works() {
+        let texts = [
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(FOO(A), A).
+            qux() ->
+                ?FOO(\"aaa\"
+                     \"bbb\"  % comment
+                     \"ccc\").
+            "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(FOO, foo).
+            qux(A) ->
+                case A of
+                    a ->
+                        ?FOO
+                %% comment
+                end.
+            "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(a(X), X X).
+            foo() ->
+                1 ?a(?a(+1)).
+            "},
+        ];
+        for text in texts {
+            crate::assert_format!(text, Module);
+        }
+    }
+
+    #[test]
+    fn macro_lazy_expand_works() {
+        let texts = [
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(BAR,
+                    ?FOO 1 end).
+            -define(FOO, begin).
+            baz() ->
+                ?BAR.
+            "},
+            indoc::indoc! {"
+            %---10---|%---20---|
+            -define(FOO,
+                    ?BAR(1)).
+            -define(BAR(X), X).
+
+            foo() ->
+                [?FOO].
+            "},
+        ];
+        for text in texts {
+            crate::assert_format!(text, Module);
+        }
+    }
+
+    #[test]
+    fn circular_macro_works() {
+        let texts = [indoc::indoc! {"
+            %---10---|%---20---|
+            -define(a, ?b).
+            -define(b, ?a).
+            foo() ->
+                ?a == ?b.
+            "}];
+        for text in texts {
+            crate::assert_format!(text, Module);
+        }
+    }
 }
