@@ -21,6 +21,7 @@ pub enum Error {
 
 impl Error {
     pub fn unexpected_token(ts: &TokenStream, token: Token) -> Self {
+        // TODO: Optimize (lazily generate the message)
         let message = Self::generate_unexpected_token_place(ts.text(), ts.filepath(), &token);
         Self::UnexpectedToken {
             position: token.start_position(),
@@ -78,7 +79,7 @@ pub trait Parse: Sized {
     fn parse(ts: &mut TokenStream) -> Result<Self>;
 }
 
-impl<A: Parse> Parse for Box<A> {
+impl<T: Parse> Parse for Box<T> {
     fn parse(ts: &mut TokenStream) -> Result<Self> {
         ts.parse().map(Box::new)
     }
@@ -89,3 +90,16 @@ impl<A: Parse, B: Parse> Parse for (A, B) {
         Ok((ts.parse()?, ts.parse()?))
     }
 }
+
+pub trait ResumeParse<A>: Parse {
+    fn resume_parse(ts: &mut TokenStream, args: A) -> Result<Self>;
+}
+
+impl<T, A> ResumeParse<A> for Box<T>
+where
+    T: ResumeParse<A>,
+{
+    fn resume_parse(ts: &mut TokenStream, args: A) -> Result<Self> {
+        ts.resume_parse(args).map(Box::new)
+    }
+}
