@@ -40,6 +40,26 @@ impl<T: Span> Span for Box<T> {
     }
 }
 
+impl Span for Position {
+    fn start_position(&self) -> Position {
+        *self
+    }
+
+    fn end_position(&self) -> Position {
+        *self
+    }
+}
+
+impl Span for std::ops::Range<Position> {
+    fn start_position(&self) -> Position {
+        self.start
+    }
+
+    fn end_position(&self) -> Position {
+        self.end
+    }
+}
+
 impl<A: Span, B: Span> Span for (A, B) {
     fn start_position(&self) -> Position {
         self.0.start_position()
@@ -50,7 +70,9 @@ impl<A: Span, B: Span> Span for (A, B) {
     }
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord)]
+#[derive(
+    Debug, Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord, serde::Serialize, serde::Deserialize,
+)]
 pub struct Position {
     offset: usize,
     line: usize,
