@@ -11,6 +11,5 @@
 
 hello(A) ->
     io?BAR "hello ~p\n", [A]).
-main() ?a(?qux a, b)
-    , c],
+main() ?a(?qux a, b), c],
     [1, hello(world?quux 3].
